/********************************************************************
  Nuomos Punktas
  Auhor: Gita Kurklietyte

  punktas.cpp
********************************************************************/

#include <iostream>
#include <fstream>

#include "rubai.h"
#include "client.h"
#include "rent.h"

using namespace std;

void printChoices();
void registerClient(ClientPtrVector& clients);
void registerRubas  (PrekePtrVector& rubai);
void rent(ClientPtrVector& clients,
          RentPtrVector& rents,
          PrekePtrVector& rubai);
void takeBack(PrekePtrVector& rubai);
void listClients(ClientPtrVector& clients);
void listRubas(PrekePtrVector& rubai);
void listRents(RentPtrVector& rents);
void loadFromFile(ClientPtrVector& clients,
                  RentPtrVector& rents,
                  PrekePtrVector& rubai);
void saveToFile(ClientPtrVector& clients,
                RentPtrVector& rents,
                PrekePtrVector& rubai);
//----------------------------------------------------------------
int main()
{
  ClientPtrVector clients;  //klientu masyvas
  RentPtrVector rents;      //nuomojimu masyvas
  PrekePtrVector rubai;     //rubu masyvas
  loadFromFile (clients,rents,rubai);
  string choice;
  for(;;)
  {
    printChoices();
    cin >> choice;
    if (choice == "1") registerClient(clients);           //registerClient
    else if (choice == "2")                               //registerRubas
    {
      try
      {
        registerRubas  (rubai);
      }
      catch (string& e){cout << e << endl;}
    }

    else if (choice == "3")                               //rent
    {
      try
      {
        rent(clients,rents,rubai);
      }
      catch (string& e){cout << e << endl;}
    }

    else if (choice == "4") listClients(clients);         //list
    else if (choice == "5") listRubas(rubai);
    else if (choice == "6") listRents(rents);
    else if (choice == "7")                               //grazinimas
    {
      try
      {
        takeBack(rubai);
      }
      catch (string& e){cout << e << endl;}
    }

    else if (choice == "0")                               //exit
    {
      saveToFile(clients,rents,rubai);
      return 0;
    }
    else cout << "Neteisingai!!!" << endl;
  }
}
//---------------------------------------------------------------
void printChoices()
{
  cout << endl
       << "1 - Registruoti klienta"    << endl
       << "2 - Registruoti ruba"       << endl
       << "3 - Nuomoti"                << endl
       << "4 - Klientai..."            << endl
       << "5 - Rubai..."               << endl
       << "6 - Nuomojimai..."          << endl
       << "7 - Registruoti grazinta ruba." << endl
       << endl
       << "0 - Exit" << endl;
}
//---------------------------------------------------------------
void registerClient(ClientPtrVector& clients)
{
  string name;
  string surname;
  cout << "Naujo kliento vardas: ";
  cin >> name;
  cout << "Naujo kliento pavarde: ";
  cin >> surname;
  clients.push_back(new Client(name,surname));
  cout << "Klientas uzregistruotas. " << endl;
}
//---------------------------------------------------------------
void registerRubas  (PrekePtrVector& rubai)
{
  string ID;
  cout << "Ivesk rubo pavadinima: ";
  cin >> ID;
  double rent;
  cout << "Valandos nuomos kaina: ";
  cin >> rent;
  string choice;
  cout << "Rubo tipas: " << endl
       << "0 - Suknele"        << endl
       << "1 - Kostiumas " << endl
       << "2 - Aksesuaras"       << endl;
  cin >> choice;
  if (choice == "0"){
    rubai.push_back(new Suknele(ID,rent));
  }
  else if (choice == "1"){
    rubai.push_back(new Kostiumas (ID,rent));
  }
  else if (choice == "2"){
    rubai.push_back(new Aksesuaras(ID,rent));
  }
  else throw string("Neteisingai pasirinkai tipa");

}
//---------------------------------------------------------------
double total4Client (RentPtrVector& rents, Client* c)
  //randam kiek konkretus klientas nuomojo is viso 
{
  double totalRented = 0;
  for (unsigned int i=0; i<rents.size(); i++)
  {
//jei nuomojime nurodytas klientas yra tas,kuriam skaiciuojam
    if (rents[i]->getClient()==c)   
    {
      totalRented += rents[i]->getTotal();
    }
  }
  return totalRented;
}
//---------------------------------------------------------------
void rent(ClientPtrVector& clients,
          RentPtrVector& rents,
          PrekePtrVector& rubai)
{
  unsigned int rubas;
  listRubas(rubai);
  cout << "Kuri ruba nuomosime? ";
  cin >> rubas;
  if (!rubai[rubas]->getRented())
  {
    unsigned int client;
    listClients(clients);
    cout << "Kuriam klientui? ";
    cin >> client;
    double durat;
    cout << "Kelioms valandoms? ";
    cin >> durat;
    Rent* r = new Rent (durat,clients[client],total4Client(rents,clients[client]),rubai[rubas]);  //sukuriam nuomojima
    rents.push_back(r);  //idedam i nuomojimu masyva
    rubai[rubas]->addRent(r); //rubui pridedam nuomojima
    rubai[rubas]->setRented(true);  //nustatom,kad jau isnuomotas
    cout << "Nuomojimas uzregistruotas. Klientui reikia moketi: " << r->getTotal() << endl;
  }
  else throw string("Bandai nuomoti jau isnuomota ruba.");
  
}
//---------------------------------------------------------------
void takeBack(PrekePtrVector& rubai)
{
  unsigned int rubas;
  cout << endl << "Isnuomota: " << endl;
  for (unsigned int i = 0; i<rubai.size(); i++)
  {
    if (rubai[i]->getRented())
    {
      cout << i << " " << rubai[i]->getID() << endl;
    }
  }
  cout << "Kuri grazino: ";
  cin >> rubas;
  if (rubas >= rubai.size())
  {
    throw string ("Toks rubas neregistruotas");
  }
  rubai[rubas]->setRented(false);// nurodom, kad rubas neisnuomotas
}
//---------------------------------------------------------------
void listClients(ClientPtrVector& clients)
{
  cout << endl << "Klientai: " << endl;
  for (unsigned int i=0; i<clients.size(); i++)
  {
    cout << i << " " << clients[i]->getName() << " " << clients[i]->getSurname() << endl;
  }
}
//---------------------------------------------------------------
void listRubas(PrekePtrVector& rubai)
{
  cout << endl << "Rubai: " << endl;
  for (unsigned int i = 0; i<rubai.size(); i++)
  {
    cout << i << " " << rubai[i]->getID() << " " << rubai[i]->getRentProHour() << endl;
  }
}
//---------------------------------------------------------------
void listRents(RentPtrVector& rents)
{
  cout << endl << "Isnuomota:" << endl;
  for (unsigned int i=0; i<rents.size(); i++)
  {
    cout <<rents[i]->getPreke()->getID() << ": nuomojo " 
         << rents[i]->getClient()->getName() << " " << rents[i]->getClient()->getSurname()
         << "  "
         << rents[i]->getDuration() << " val. uz " 
         << rents[i]->getTotal() << endl;
  }
}
//---------------------------------------------------------------
Client* findClient (ClientPtrVector& clients,string name,string surname)
{
  for (unsigned int i=0; i<clients.size(); i++)
  {
    if (clients[i]->getName()==name&&clients[i]->getSurname()==surname)
    {
      return clients[i];
    }
  }
  return 0;
}
//---------------------------------------------------------------
Preke* findPreke (PrekePtrVector& rubai,string ID)
{
  for (unsigned int i = 0; i<rubai.size(); i++)
  {
    if (rubai[i]->getID()==ID) return rubai[i];
  }
  return 0;
}
//---------------------------------------------------------------
void loadFromFile(ClientPtrVector& clients,
                  RentPtrVector& rents,
                  PrekePtrVector& rubai)
{
  int i;
  ifstream in ("book.txt");
  int clientCount,rubasCount,rentCount;
  in >> clientCount;
  string name,surname;
  for (i = 0; i<clientCount; i++)
  {
    in >> name;
    in >> surname;
    clients.push_back(new Client (name,surname));
  }

  string ID;
  double rent;
  int rented;
  in >> rubasCount;
  for (i = 0; i<rubasCount; i++)
  {
    in >> ID;
    in >> rent;
    in >> rented;
    Rubas* suknele = new Suknele (ID,rent);
    suknele->setRented(rented);
    rubai.push_back(suknele);
  }

  double duration;
  in >> rentCount;
  for (i = 0; i< rentCount; i++)
  {
    in >> ID;
    in >> name;
    in >> surname;
    in >> duration;
    rents.push_back(new Rent(duration,
                             findClient(clients,name,surname),
                             total4Client(rents,findClient(clients,name,surname)),
                             findPreke  (rubai,ID)));
  }
}
//---------------------------------------------------------------
void saveToFile(ClientPtrVector& clients,
                RentPtrVector& rents,
                PrekePtrVector& rubai)
{
  int i;
  ofstream out ("book.txt");
  out << clients.size() << endl;
  for (i=0; i<clients.size(); i++)
  {
    out << clients[i]->getName() << endl << clients[i]->getSurname() << endl;
  }

  out << rubai.size() << endl;
  for (i = 0; i<rubai.size(); i++)
  {
    out <<rubai[i]->getID() << endl << rubai[i]->getRentProHour() << endl
        << rubai[i]->getRented() << endl;
  }
  
  out << rents.size() << endl;
  for (i=0; i<rents.size(); i++)
  {
    out << rents[i]->getPreke  ()->getID() << endl
         << rents[i]->getClient()->getName() << endl << rents[i]->getClient()->getSurname()
         << endl
         << rents[i]->getDuration() << endl;
  }
  out.close();
}

