/********************************************************************
  Nuomos Punktas
  Auhor: Jonas Norgela

  rubas.h
********************************************************************/

#ifndef RUBAS_H
#define RUBAS_H

#include <vector>
#include <string>
class Rent;
class Client;

typedef std::vector<Rent*> RentPtrVector;  //vektorius is nuorodu i nuomojimas


//virtual - nurodo, kad paveldejusiose klasese metodas gali buti perrasytas
//=0 nurodo, kad metodas yra svariai virtualus (pure virtual), t.y. sioje klaseje
//neturi realizacijos

class Preke  //sablonas, nes nera duomenu ir visi metodai svariai virtualus
{
  public:              
    virtual std::string getID() const                        = 0;
    virtual double getRentProHour() const                    = 0;
    virtual void setRentProHour(double r)                    = 0;
    virtual bool getRented()                                 = 0;
    virtual void setRented(bool r)                           = 0;
    //kiekviena preke tures saugot nuomojimu masyva 
    virtual RentPtrVector getRents()const                    = 0;
    virtual void addRent (Rent* r)                           = 0;
    virtual double priceWithDiscount (double duration,double total) = 0;
    virtual ~Preke(){}
};

class Rubas: public Preke //abstrakti klase, nes turi svariai virtualu metoda
{
  private:
    std::string ID;
    double rentProHour;
    bool rented;
    RentPtrVector rents; //nuomojimu vektorius (masyvas)
  public:
    Rubas   (std::string ID,double rentProHour);
    virtual std::string getID() const {return ID;}
    virtual double getRentProHour() const {return rentProHour;}
    virtual void setRentProHour(double r) {rentProHour=r;}
    virtual bool getRented(){return rented;}
    virtual void setRented(bool r){rented=r;}
    virtual RentPtrVector getRents()const {return rents;}
    virtual void addRent (Rent* r){rents.push_back(r);}
    virtual double priceWithDiscount (double duration,double total) = 0;
};
typedef std::vector<Preke*> PrekePtrVector;
#endif

